@@ -1,5 +1,7 @@
 import time
+import sys
 import logging
+import traceback
 import tkinter as tk
 from tkinter import PhotoImage, ttk
 from multiprocessing import Queue
@@ -15,6 +17,7 @@ from .levels import LevelsTab
 from .pack import PackTab
 from .widgets import ConsoleWindow
 from .logs import QueueHandler, register_queue_handler
+from .error import ErrorTab
 
 logger = logging.getLogger("modlunky2")
 
@@ -23,10 +26,14 @@ MIN_WIDTH = 1280
 MIN_HEIGHT = 900
 
 
+def exception_logger(type_, value, traceb):
+    exc = traceback.format_exception(type_, value, traceb)
+    logger.critical("Unhandled Exception: %s", "".join(exc).strip())
+
+
 class ModlunkyUI:
-    def __init__(self, config, data_dir):
+    def __init__(self, config, log_level=logging.INFO):
         self.config = config
-        self.data_dir = data_dir
 
         self.current_version = current_version()
         self.latest_version = latest_version()
@@ -44,8 +51,8 @@ class ModlunkyUI:
 
         self.log_queue = Queue()
         self.queue_handler = QueueHandler(self.log_queue)
-        register_queue_handler(self.queue_handler)
-        self.task_manager = TaskManager(self.log_queue)
+        register_queue_handler(self.queue_handler, log_level)
+        self.task_manager = TaskManager(self.log_queue, log_level)
 
         self.root = tk.Tk(className="Modlunky2")  # Equilux Black
         self.root.title("Modlunky 2")
@@ -59,6 +66,9 @@ class ModlunkyUI:
         self.icon_png = PhotoImage(file=BASE_DIR / "static/images/icon.png")
         self.root.iconphoto(False, self.icon_png)
 
+        sys.excepthook = exception_logger
+        self.root.report_callback_exception = exception_logger
+
         if self.needs_update:
             update_frame = ttk.LabelFrame(self.root, text="Modlunky2 Update Available")
             update_frame.grid(row=0, column=0, sticky="nswe")
@@ -76,40 +86,38 @@ class ModlunkyUI:
 
         # Handle shutting down cleanly
         self.root.protocol("WM_DELETE_WINDOW", self.quit)
+        self.root.bind("<Escape>", self.quit)
 
         self.tabs = {}
         self.tab_control = ttk.Notebook(self.root)
 
         self.register_tab(
             "Pack Assets",
-            PackTab(
-                tab_control=self.tab_control,
-                config=config,
-                task_manager=self.task_manager,
-            ),
+            PackTab,
+            tab_control=self.tab_control,
+            config=config,
+            task_manager=self.task_manager,
         )
         self.register_tab(
             "Extract Assets",
-            ExtractTab(
-                tab_control=self.tab_control,
-                config=config,
-                task_manager=self.task_manager,
-            ),
+            ExtractTab,
+            tab_control=self.tab_control,
+            config=config,
+            task_manager=self.task_manager,
         )
         self.register_tab(
             "Level Editor",
-            LevelsTab(
-                tab_control=self.tab_control,
-                config=config,
-            ),
+            LevelsTab,
+            tab_control=self.tab_control,
+            modlunky_ui=self,
+            config=config,
         )
 
         self.register_tab(
             "Config",
-            ConfigTab(
-                tab_control=self.tab_control,
-                config=config,
-            ),
+            ConfigTab,
+            tab_control=self.tab_control,
+            config=config,
         )
 
         self.tab_control.bind("<<NotebookTabChanged>>", self.on_tab_change)
@@ -169,19 +177,62 @@ class ModlunkyUI:
         tab_name = event.widget.tab("current")["text"]
         tab = self.tabs[tab_name]
         if tab.show_console:
-            self.console_frame.grid(row=2, column=0, padx=5, pady=(5, 0), sticky="nswe")
+            self.render_console()
         else:
-            self.console_frame.grid_forget()
+            self.forget_console()
         tab.on_load()
 
-    def register_tab(self, name, obj):
+    def render_console(self):
+        self.console_frame.grid(row=2, column=0, padx=5, pady=(5, 0), sticky="nswe")
+
+    def forget_console(self):
+        self.console_frame.grid_forget()
+
+    def register_tab(self, name, cls, **kwargs):
+        try:
+            obj = cls(**kwargs)
+        except Exception:  # pylint: disable=broad-except
+            obj = ErrorTab(tab_control=self.tab_control)
+            logger.critical(
+                "Failed to register tab %s: %s",
+                name,
+                "".join(traceback.format_exception(*sys.exc_info())).strip(),
+            )
+
         self.tabs[name] = obj
         self.tab_control.add(obj, text=name)
 
-    def quit(self):
+    def tabs_needing_save(self):
+        needs_save = []
+        for tab_name, tab in self.tabs.items():
+            if tab.save_needed:
+                needs_save.append(tab_name)
+        return needs_save
+
+    def should_close(self, needs_save):
+        tabs = ", ".join(needs_save)
+        msg_box = tk.messagebox.askquestion(
+            "Close Modlunky2?",
+            (
+                f"You have some tabs ({tabs}) with unsaved changes.\n"
+                "Are you sure you want to exit without saving?"
+            ),
+            icon="warning",
+        )
+        return msg_box == "yes"
+
+    def quit(self, _event=None):
         if self._shutting_down:
             return
 
+        needs_save = self.tabs_needing_save()
+        should_close = True
+        if needs_save:
+            should_close = self.should_close(needs_save)
+
+        if not should_close:
+            return
+
         self._shutting_down = True
         logger.info("Shutting Down.")
         for handler in self._shutdown_handlers:
